﻿#include "StdAfx.h"
#include ".\simpletextshape.h"
#include <GdiPlus.h>
#include "GDIPlusDC.h"
#include "ScriptletConstant.h"

REGISTER_COMPONENT_CREATOR(SCRIPTLET_SIMPLETEXT_NODE, CSimpleTextShape);
IMPLEMENT_RTTI(CSimpleTextShape, CDrawingShape);
CSimpleTextShape::CSimpleTextShape(void)
{
	m_cf.dwMask = CFM_BOLD | CFM_ITALIC | CFM_FACE | CFM_CHARSET | CFM_COLOR | CFM_BACKCOLOR | CFM_SIZE;
	m_cf.dwEffects = CFE_AUTOBACKCOLOR;
	_tcsncpy(m_cf.szFaceName, _T("굴림"), LF_FACESIZE);
	m_cf.bCharSet = DEFAULT_CHARSET;
	m_cf.crTextColor = RGB(0x0, 0x0, 0x0);
	m_cf.crBackColor = RGB(0xff, 0xff, 0xff);
	m_cf.yHeight = 10 * 20;

	m_pf.dwMask = PFM_ALIGNMENT;
	m_pf.wAlignment = PFA_LEFT;

	m_nVAlign = PFA_TOP;
	m_bWordbreak = FALSE;
	m_bTrimming = FALSE;
}

CSimpleTextShape::~CSimpleTextShape(void)
{
}

void CSimpleTextShape::DrawComponent(HDC hDC)
{
	CRect rcClient;
	GetDrawingBounds(&rcClient);
	
	if ((m_cf.dwEffects & CFE_AUTOBACKCOLOR) == 0)
	{
		FillSolidRect(hDC, &rcClient, m_cf.crBackColor);
	}
	UINT nFormat = 0;

	if (m_bWordbreak)
		nFormat = DT_WORDBREAK;

	switch (m_nVAlign)
	{
	case PFA_TOP: nFormat |= DT_TOP; break;
	case PFA_VCENTER: nFormat |= DT_VCENTER; break;
	case PFA_BOTTOM: nFormat |= DT_BOTTOM; break;
	}
	switch (m_pf.wAlignment)
	{
	case PFA_LEFT: nFormat |= DT_LEFT; break;
	case PFA_CENTER: nFormat |= DT_CENTER; break;
	case PFA_RIGHT: nFormat |= DT_RIGHT; break;
	}

	// ...을 지원
	if( m_bTrimming )
	{
		nFormat |= DT_END_ELLIPSIS;
	}

	CGDIPlusDC dc(hDC);
	dc.DrawText(m_strText, _tcslen(m_strText), &rcClient, nFormat, m_cf.szFaceName, m_cf.yHeight / 20, (m_cf.dwEffects & CFE_BOLD), 
				(m_cf.dwEffects & CFE_ITALIC), m_cf.dwEffects & CFE_UNDERLINE, m_cf.crTextColor);
}
void CSimpleTextShape::SetText(LPCTSTR pszcText)
{
	m_strText = pszcText;
}
void CSimpleTextShape::SetCharFormat(CHARFORMAT2& cf)
{
	CopyMemory(&m_cf, &cf, sizeof(CHARFORMAT2));
}
void CSimpleTextShape::SetParaFormat(PARAFORMAT2& pf)
{
	CopyMemory(&m_pf, &pf, sizeof(PARAFORMAT2));
}
void CSimpleTextShape::SetFontName(LPCTSTR pszcFontName, BYTE bCharset)
{
	_tcsncpy(m_cf.szFaceName, pszcFontName, LF_FACESIZE);
	m_cf.bCharSet = bCharset;
}
LPCTSTR CSimpleTextShape::GetFontname()
{
	return m_cf.szFaceName;
}
BYTE CSimpleTextShape::GetCharset()
{
	return m_cf.bCharSet;
}
void CSimpleTextShape::SetFontSize(int nFontSize)
{
	m_cf.yHeight = nFontSize * 20;
}
int CSimpleTextShape::GetFontSize()
{
	return m_cf.yHeight / 20;
}
void CSimpleTextShape::SetFontColor(COLORREF clrFont)
{
	m_cf.crTextColor = clrFont;
}
COLORREF CSimpleTextShape::GetFontColor()
{
	return m_cf.crTextColor;
}
void CSimpleTextShape::SetBackground(COLORREF clrBack)
{
	m_cf.crBackColor = clrBack;
}
COLORREF CSimpleTextShape::GetBackground()
{
	return m_cf.crBackColor;
}
void CSimpleTextShape::SetUnderline( BOOL bUnderLine )
{
	SetFlag(bUnderLine, m_cf.dwEffects, CFE_UNDERLINE);
}
void CSimpleTextShape::SetBold(BOOL bBold)
{
	SetFlag(bBold, m_cf.dwEffects, CFE_BOLD);
}
BOOL CSimpleTextShape::IsBold()
{
	return GetFlag(m_cf.dwEffects, CFE_BOLD);
}
void CSimpleTextShape::SetItalic(BOOL bItalic)
{
	SetFlag(bItalic, m_cf.dwEffects, CFE_ITALIC);
}
BOOL CSimpleTextShape::IsItalic()
{
	return GetFlag(m_cf.dwEffects, CFE_ITALIC);
}
void CSimpleTextShape::SetOutline(BOOL bOutline)
{
	SetFlag(bOutline, m_cf.dwEffects, CFE_OUTLINE);
}
BOOL CSimpleTextShape::IsOutline()
{
	return GetFlag(m_cf.dwEffects, CFE_OUTLINE);
}
void CSimpleTextShape::SetVAlign(int nAlign)
{
	m_nVAlign = nAlign;
}
int CSimpleTextShape::GetVAlign()
{
	return m_nVAlign;
}
void CSimpleTextShape::SetHAlign(int nAlign)
{
	m_pf.wAlignment = nAlign;
}
int CSimpleTextShape::GetHAlign()
{
	return m_pf.wAlignment;
}

BOOL CSimpleTextShape::GetOrginPreferredSize( LPSIZE lpSize )
{
	CSize sPreferred(0, 0);
	BOOL bSuccess = __super::GetOrginPreferredSize(&sPreferred);
	if ( m_strText.GetLength() > 0 )
	{
		CSize sTextPreferred(sPreferred.cx, sPreferred.cy);
		if (GetTextPreferredSize(&sTextPreferred))
		{
			sPreferred.cx = max(sPreferred.cx, sTextPreferred.cx);
			sPreferred.cy = max(sPreferred.cy, sTextPreferred.cy);
			bSuccess = TRUE;
		}		
	}	
	lpSize->cx = sPreferred.cx;
	lpSize->cy = sPreferred.cy;
	return bSuccess;
}
BOOL CSimpleTextShape::GetTextPreferredSize(LPSIZE lpSize)
{
	if ( m_strText.GetLength() == 0 ) return FALSE;

	CSize sPreferred(0, 0);
	HDC hDC = ::GetDC(NULL); 
	if (hDC != NULL)
	{
// 		UINT nFormat = DT_EDITCONTROL | DT_WORDBREAK;
// 		switch (m_nVAlign)
// 		{
// 		case PFA_TOP: nFormat |= DT_TOP; break;
// 		case PFA_VCENTER: nFormat |= DT_VCENTER; break;
// 		case PFA_BOTTOM: nFormat |= DT_BOTTOM; break;
// 		}
// 		switch (m_pf.wAlignment)
// 		{
// 		case PFA_LEFT: nFormat |= DT_LEFT; break;
// 		case PFA_CENTER: nFormat |= DT_CENTER; break;
// 		case PFA_RIGHT: nFormat |= DT_RIGHT; break;
// 		}

		CRect rcClient;
		GetClientRect(&rcClient);
		RectF rcText(0, 0, lpSize->cx, lpSize->cy);
		CGDIPlusDC dc(hDC);
		dc.MeasureString(m_strText, m_cf.szFaceName, m_cf.yHeight / 20, (m_cf.dwEffects & CFE_BOLD), 
			(m_cf.dwEffects & CFE_ITALIC), rcText);
		::ReleaseDC(NULL, hDC);

		if (sPreferred.cx == 0)
		{
			sPreferred.cx = static_cast<LONG>(rcText.Width) + 1;
		}
		if (sPreferred.cy == 0)
		{
			sPreferred.cy = static_cast<LONG>(rcText.Height);
		}
	}	
	lpSize->cx = sPreferred.cx;
	lpSize->cy = sPreferred.cy;
	return TRUE;
}